package com.me.brickbuster.entity;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.PolygonRegion;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.EarClippingTriangulator;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.utils.ShortArray;
import com.me.brickbuster.entity.powerup.PowerUpType;
import com.me.brickbuster.physics.CollisionListener;
import com.me.brickbuster.physics.EntityType;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.PlayState;

public class Brick extends Entity implements PhysicsBody, CollisionListener {

	public static final Color DEFAULT_COLOR = Color.FOREST;
	public static final float BRICK_WIDTH = 5f;
	public static final float BRICK_HEIGHT = 2.5f;

	private static final EarClippingTriangulator ECT = new EarClippingTriangulator();

	private BrickType type;
	private BrickShape shape;
	private PowerUpType powerUpType;

	private Color color;
	private Pixmap pm;
	private Texture solid;
	private TextureRegion region;

	private Body body;
	private boolean hitByBall = false;

	private final Vector2 tmp = new Vector2();

	public Brick(PlayState state, PowerUpType powerUpType, float x, float y) {
		this(state, powerUpType, true, x, y);
	}

	public Brick(PlayState state, BrickShape shape, PowerUpType powerUpType, float x, float y) {
		this(state, BrickType.STANDARD_10, shape, powerUpType, DEFAULT_COLOR, true,     x, y);
	}

	public Brick(PlayState state, PowerUpType powerUpType, boolean hidePowerup, float x, float y) {
		this(state, BrickType.STANDARD_10, BrickShape.RECTANGLE, powerUpType, DEFAULT_COLOR, hidePowerup, x, y);
	}

	public Brick(PlayState state, BrickType type, BrickShape shape, PowerUpType powerUpType, Color color, boolean hidePowerUp, float x, float y) {
		super(state, x, y);
		this.type = type;
		this.shape = shape;
		this.powerUpType = powerUpType;
		this.color = powerUpType != null && !hidePowerUp? powerUpType.getColor() : color;
		this.pm = new Pixmap(1,1, Pixmap.Format.RGBA8888);
		pm.setColor(color);
		pm.fill();
		this.solid = new Texture(pm);
		this.region = new TextureRegion(solid);
		createBody();
	}

	@Override
	public void render(ShapeRenderer sr) {
		PolygonShape shape = (PolygonShape) body.getFixtureList().get(0).getShape();
		float[] vertices = new float[shape.getVertexCount()*2];
		for (int i = 0; i < vertices.length/2; i++) {
			shape.getVertex(i, tmp);
			Vector2 vertex = body.getWorldPoint(tmp);
			vertices[i*2] = vertex.x * PlayState.PIXEL_PER_METER;
			vertices[i*2 + 1] = vertex.y * PlayState.PIXEL_PER_METER;
		}
		ShortArray triangleIndices = ECT.computeTriangles(vertices);

		PolygonRegion polyRegion = new PolygonRegion(region, vertices, triangleIndices.toArray());

		PolygonSpriteBatch pb = state.getGame().pb;
		pb.begin();
		pb.draw(polyRegion, 0, 0);
		pb.end();
	}

	@Override
	public void update(float dt) {
		if (hitByBall) {
			hit();
			hitByBall = false;
		}
	}

	@Override
	public Body getBody() {
		return body;
	}

	@Override
	public void createBody() {
		BodyDef brickBody = new BodyDef();
		brickBody.type = BodyDef.BodyType.StaticBody;
		brickBody.position.set(pos.cpy());

		PolygonShape brickShape = new PolygonShape();

		switch (shape) {
			case DIAMOND:
				brickShape.set(new float[] {
						-BRICK_WIDTH/2, 0,  // Left
						0, BRICK_HEIGHT/2,  // UP
						BRICK_WIDTH/2, 0,   // Right
						0, -BRICK_HEIGHT/2  // Down
				});
				break;
			default:
				brickShape.setAsBox(BRICK_WIDTH/2, BRICK_HEIGHT/2, Vector2.Zero, 0f);
		}


		FixtureDef brickFixture = new FixtureDef();
		brickFixture.shape = brickShape;
		brickFixture.friction = 0f;
		brickFixture.density = 0.5f;

		brickFixture.filter.categoryBits = EntityType.BRICK;
		brickFixture.filter.maskBits = EntityType.BALL | EntityType.BRICK | EntityType.BOUNDARY;

		body = state.world.createBody(brickBody);
		body.createFixture(brickFixture);
		body.setUserData(this);

		brickShape.dispose();
	}

	@Override
	public void beginContact(Entity contacted) {
		if (contacted instanceof Ball) {
			hitByBall = true;
		}
	}

	@Override
	public void endContact(Entity contacted) {
	}

	public boolean hit() {
		if (state.bricks.size-1 <= Ball.BLOCKS_FOR_BOOST) {
			for (Ball ball : state.balls) {
				ball.setSpeed(Ball.BOOST_SPEED);
			}
		}

		if (powerUpType != null) {
			state.powerUps.add(powerUpType.createInstance(state, this));
		}

		deleted = true;
		return true;
	}

}
